#include <stdio.h>
#include <alloc.h>
#include <process.h>
#include <stdlib.h>
#include <io.h>
#include <dir.h>
#include <string.h>
#include "cp.h"

char diret_origem [100]="",
     diret_destino [100]="";
int  origem_utilizado=0,
     destino_utilizado=0,
     barra_origem=0,
     barra_destino=0;

/***************************************************************************
   Coloca uma struct arquivo em uma pilha.
   Recebe um ponteiro para um nodo e um outro para o inicio da pilha.
 ***************************************************************************/

void *poe_na_pilha (struct arquivo *ini_pilha, struct arquivo *nodo_corr)
{
   nodo_corr->prox=ini_pilha==NULL?NULL:ini_pilha;
   return nodo_corr;
}

/***************************************************************************
   Pega todas as entradas no diretorio que combinam com a restricao forneci-
   da e acrescenta ao topo da pilha.
   Recebe um ponteiro para string com a restricao e um ponteiro para struct
   arquivo contendo o topo da pilha.
 ***************************************************************************/

void *pega_arqs (char *restricao, struct arquivo *ini_pilha)
{
  struct arquivo *nodo;  /* Nodo contendo as informacoes do arquivo */
  struct ffblk item_dir; /* Estrutura de carga de inform.do arquivo */
  int erro,
      atributo,
      japassou=0; /* Inicializar a leitura do dir. Contar as entradas.*/
  char selec_arq [100]="";

  strcat (selec_arq, diret_origem);
  if (origem_utilizado)
    if (!barra_origem)
      strcat (selec_arq, "\\");
  strcat (selec_arq, restricao);
  do {
    if (!japassou) /* Primeira leitura */
      erro=findfirst (selec_arq, &item_dir, atributo);
    else           /* Proxima leitura */
      erro=findnext (&item_dir);
    if (!erro) {
      japassou++;
      nodo=malloc(sizeof(struct arquivo));
      if (nodo==NULL) {
        tira_pilha_dir (ini_pilha);
        return NULL;
      }
      strcpy (nodo->nome, item_dir.ff_name);
      nodo->tam=item_dir.ff_fsize;
      ini_pilha=poe_na_pilha (ini_pilha, nodo);
    }
  } while (!erro);
  if (!japassou)
    printf ("\nNo foi encontrado arquivo com restricao %s.\n",restricao);
  return ini_pilha;
}

/***************************************************************************
   Desaloca a pilha.
   Recebe um ponteiro para o topo da pilha.
 ***************************************************************************/

void tira_pilha_dir (struct arquivo *ini_pilha)
{
  struct arquivo *nodo_corr;

  while (ini_pilha!=NULL) {
    nodo_corr=ini_pilha->prox;
    free (ini_pilha);
    ini_pilha=nodo_corr;
  }
}

/***************************************************************************
   Carrega o arquivo especificado pela struct arquivo e o coloca numa area
   continua de memoria.
   Recebe um ponteiro para a struct arquivo contendo as informacoes do arqui-
   vo que sera carregado.
   Retorna um ponteiro huge para o arquivo na memoria.
 ***************************************************************************/

char huge *carrega_arquivo (struct arquivo *identif)
{
  FILE *arq;
  char huge *arq_item, nome_arq [100]="";
  int caracter;
  long int indice;

  strcat (nome_arq, diret_origem);
  if (origem_utilizado)
    if (!barra_origem)
      strcat (nome_arq, "\\");
  strcat (nome_arq, identif->nome);
  if ((arq=fopen (nome_arq,"rb"))==NULL)
    return NULL;
  arq_item=(char huge *)farmalloc (identif->tam);
  if (arq_item==NULL)
    return NULL;
  for (indice=0; indice!=identif->tam; indice++) {
    caracter=fgetc(arq);
    arq_item[indice]=(char)caracter;
  }
  fclose (arq);
  return arq_item;
}

/***************************************************************************
   Escreve o arquivo contido na struct arquivo e grava no drive especificado.
   Retorna NULL se nao conseguiu abrir o arquivo.
 ***************************************************************************/

void escr_arq (struct arquivo *identif)

{
  FILE *arq;
  char huge *arq_item, nome_arq [100]="";
  int caracter;
  long indice;

  strcat (nome_arq, diret_destino);
  if (destino_utilizado)
    if (!barra_destino)
      strcat (nome_arq, "\\");
  strcat (nome_arq, identif->nome);
  if ((arq=fopen (nome_arq, "wb"))==NULL)
    return;
  for (indice=0; indice!=identif->tam; indice++) {
    caracter=identif->aloc[indice];
    fputc(caracter,arq);
  }
  fclose (arq);
}

/***************************************************************************
   Controla toda a parte de leitura, gravacao e controle de area livre de
   memoria.
   Recebe um ponteiro para uma struct arquivo contendo o topo da pilha.
   Retorna o numero de arquivo copiados.
 ***************************************************************************/

int copia_arqs (struct arquivo *ini_pilha)

{
  struct arquivo *nodo_corr,
		 *carr_arq;
  int no_arqs=0;

  carr_arq=nodo_corr=ini_pilha;
  do {
    if (carr_arq!=NULL) {
      printf ("Lendo %s\n",carr_arq->nome);
      carr_arq->aloc=carrega_arquivo (carr_arq);
    }
    if (carr_arq==NULL || carr_arq->aloc==NULL) {
      printf ("\nPrepare-se para gravar.\nTecle enter:\n\n");
      getch ();
      while (ini_pilha!=carr_arq) {
	printf ("Arquivo: %s Tamanho: %lu\n",ini_pilha->nome,ini_pilha->tam);
	escr_arq (ini_pilha);
	no_arqs++;
	ini_pilha=ini_pilha->prox;
	farfree (nodo_corr->aloc);
	free (nodo_corr);
	nodo_corr=ini_pilha;
      }
      if (carr_arq!=NULL) {
	printf ("\nPrepare-se para ler.\nTecle enter:\n\n");
	getch ();
      }
    }
    else
      carr_arq=carr_arq->prox;
  } while (ini_pilha!=NULL);
  return no_arqs;
}

/***************************************************************************
   Recebe por parametro os arquivos que devem ser copiados. Se nao houver
   parametros, entao mostra como se deve utilizar o programa.
 ***************************************************************************/

void main (int argc, char *argv[])
{
  struct arquivo *arq_fonte=NULL;
  int i=1, tam;

  if (argc<2) {
    printf ("\nCopia arquivos usando um ou mais dispositivos de saida.\n");
    printf ("Usar cp [opcoes] fontes ...\n");
    printf ("Opcoes: [-Odiretorio_origem] [-Ddiretorio_destino]\n");
    printf ("Pode-se usar \"?\" e \"*\"\n");
    exit (0);
  }
  for (i=1; argv[i]; i++)
    if (argv[i][0]=='-') {
      tam=strlen (argv[i]);
      switch (argv[i][1]) {
	case ('o'):
	case ('O'):
	  origem_utilizado=1;
	  strcpy (diret_origem, argv[i]+2);
	  if (argv[i][tam-1]=='\\')
	    barra_origem=1;
	  break;
	case ('d'):
	case ('D'):
	  destino_utilizado=1;
	  strcpy (diret_destino, argv[i]+2);
	  if (argv[i][tam-1]=='\\')
	    barra_destino=1;
	  break;
      }
    }
    else
      arq_fonte=pega_arqs (argv[i],arq_fonte);
  if (arq_fonte)
    printf ("\n%d arquivo(s) copiado(s).\n",copia_arqs (arq_fonte));
}
